import 'package:rxdart/rxdart.dart';
// ignore: implementation_imports
import 'package:rxdart/src/transformers/backpressure/backpressure.dart';

extension StreamDartExt<T> on Stream<T> {
  Stream<T> distinctType() => distinct((bef, aft) => bef.runtimeType == aft.runtimeType);
  Stream<T> dumpErrorToConsoleDart() => doOnError((error, stackTrace) {
        print(error);
        print(stackTrace);
      });

  Stream<List<T>> bufferWithTimer(Duration duration, {bool ignoreEmpty = true}) =>
      advBuffer((_) => TimerStream<bool>(true, duration), ignoreEmpty: ignoreEmpty);

  Stream<List<T>> advBuffer(
    Stream<dynamic> Function(T event) streamFactory, {
    WindowStrategy strategy = WindowStrategy.everyEvent,
    bool ignoreEmpty = true,
  }) =>
      transform(BackpressureStreamTransformer(
        strategy,
        streamFactory,
        onWindowEnd: (queue) => queue,
        ignoreEmptyWindows: ignoreEmpty,
      ));

  Stream<T> advDebounce(
    Stream<dynamic> Function(T event) streamFactory, {
    WindowStrategy strategy = WindowStrategy.everyEvent,
    bool ignoreEmpty = true,
  }) =>
      transform(BackpressureStreamTransformer(
        strategy,
        streamFactory,
        onWindowEnd: (queue) => queue.last,
        ignoreEmptyWindows: ignoreEmpty,
      ));
}

extension FutureDartExt<T> on Future<T> {
  Future<T> dumpErrorToConsoleDart() => catchError((error, stackTrace) {
        print(error);
        print(stackTrace);
      });
}
